function [fig, hndls] = plotMesh(in1,t,options)
%% plotMesh: plot [p e t] mesh (+ quality of mesh|marks selected elements)
% -utility-
%
%  Syntax:
%    [fig hndls] = plotMesh(in1,t,options);
%
%  Examples:
%    plotMesh(p,t);
%    plotMesh(pTCMout);
%    figHandle = plotMesh(pTCMout);
%    [figHndl hndls] = plotMesh(p,t);
%    plotMesh(p,t,opt);
%
%__________________________________________________________________________
% INPUT: (ver.1, total nargin == 1 || 3)
%     pTCMout  ~ pTCMout promnn (nezbytn)
%     []       ~ przdn pole
%     options  ~ nastaven (seznam viz ne)
%__________________________________________________________________________
% INPUT: (ver.2a, total nargin == 2 || 3)
%     p        ~ matice p, obsahuje body me (nezbytn)
%     t        ~ matice t, obsahuje spojnice bod (nezbytn)
%     options  ~ nastaven (seznam viz ne)
% -------------------------------------------------------------------------
% [...]| .fsize       (size and posit. of figure, standard: [100 100 947 737])
% [...]| .ftag        (tag of plotEigNum figure)
% [RGB]| .FaceClr     (color of the faces)
% [RGB]| .EdgeClr     (color of the edges)
% [RGB]| .FaceUser    (face color of the tagged elements)
% [RGB]| .EdgeUser    (edge color of the tagged elements)
%   1  | .LineWth     (line width)
%      | .figureName  (any string)
%      | .tr          (tagged triangles)
%      | .pt          (tagged points [p1 p2 ...]);
%      | .edg         (tagged edges [edg1 edg2 edg3 ...]), pTCMout only;
%      | .q           (column vector of quality of triangles)
%
% -------------------------------------------------------------------------
%
% OUT: fig   (figure handle)
%      hndls (hndls.fill, hndls.trTexts, hndls.ptTexts, hndls.edgTexts,
%             hndls.qualText ~ next handles)
% -------------------------------------------------------------------------
%
% plotMesh version history:
%    ver. 1.0a (28.4.2011)
%         init version
%    ver. 2.0a (5.5.2011)
%         plot quality of mesh
%         new property (q(uality), tr in options, clims)
%    ver. 2.0b
%         'tr' property has been changed about 'tr'
%         new property edg in options
%         'FaceUser' has been changed about 'FaceUser'
%         new property EdgeUser in options
%    ver. 2.0c (9.8.2011)
%         new property pt in options
%         edg option has been changed
%
%
%    Last update: 9.8.2011
%
% Notes:
% A) In case of opt.tr, opt.pt or opt.edg, tr/pt/edg = 0 could be used.
%    Then all tr/pt/edg will be displayed.
%
% B) opt.edg could be used only for pTCMout input!
%
% Author(s): Miloslav apek, capekm6@fel.cvut.cz, 2011-2012
%
% See also meshComsolFcn, plotEigNum, plotEigCur

version = '2.0c';
fig     = [];
hndls   = [];

%% (0) Check input data
if nargin < 1
   fprintf(2,'Error: plotMesh failed! (pTCMout or [p,t] are necessary) \n');
   help plotMesh;
   return
end
if isfield(in1,'def') && isfield(in1.def,'type') && ...
      (strcmp(in1.def.type,'pTCMout') || strcmp(in1.def.type,'pTCMin')) ...
      && isfield(in1,'meshStruct') && ...
      isfield(in1.meshStruct,'p') && isfield(in1.meshStruct,'t')
   p = in1.meshStruct.p;
   t = in1.meshStruct.t;
   verType = 1;
elseif nargin < 2
   fprintf(2,'Error: plotMesh failed! (pTCMout or [p,t] are necessary) \n');
   help plotMesh;
   return
else
   p = in1;
   verType = 2;
end

%% (1) Check input options
if nargin < 3 || ~isfield(options,'fsize')       % fsize
   figsize = [50 100 600 500];
else
   monitor = get(0,'Screensize');
   if any(options.fsize(1:2) < 0) || any(options.fsize(3:4) <= 0) || ...
         options.fsize(1)-options.fsize(3) > monitor(3) || ...
         options.fsize(2)-options.fsize(4) > monitor(4)
      figsize = [50 100 600 500];
   else
      figsize = options.fsize;
   end
end
if nargin < 3 || ~isfield(options,'ftag') || isempty(options.ftag) % ftag
   ftag = 'plotMesh_FIG';
else
   ftag = options.ftag;
end

if nargin < 3 || ~isfield(options,'figureName') || ~ischar(options.figureName)
   figName = [];
else
   figName = options.figureName;
end
if nargin < 3 || ~isfield(options,'FaceClr') || ...   FaceColor
      ~all(size(options.FaceClr) == [1 3]) || ...
      any(options.FaceClr) > 1 || any(options.FaceClr) < 0
   options.FaceClr = [.9 .9 .9];
end
if nargin < 3 || ~isfield(options,'EdgeClr') || ...   EdgeColor
      ~all(size(options.EdgeClr) == [1 3]) || ...
      any(options.EdgeClr) > 1 || any(options.EdgeClr) < 0
   options.EdgeClr = [1 0 0];
end
if nargin < 3 || ~isfield(options,'FaceUser') || ...   FaceColor of select.
      ~all(size(options.FaceUser) == [1 3]) || ...
      any(options.FaceUser) > 1 || any(options.FaceUser) < 0
   options.FaceUser = abs(options.FaceClr-0.5);
end

if nargin < 3 || ~isfield(options,'EdgeUser') || ...   EdgeColor of select.
      ~all(size(options.EdgeUser) == [1 3]) || ...
      any(options.EdgeUser) > 1 || any(options.EdgeUser) < 0
   options.EdgeUser = abs(options.EdgeClr-0.5);
end
if nargin < 3 || ~isfield(options,'LineWth')          % width
   options.LineWth = 1;
end

if nargin < 3 || ~isfield(options,'tr')               % triangl. with tag
   tr = [];
else
   if options.tr == 0
      tr = 1:size(t,2);
   else
      tr = unique(options.tr);
   end
end
if nargin < 3 || ~isfield(options,'pt')               % points with tag
   pt = [];
else
   if options.pt == 0
      pt = 1:size(p,2);
   else
      pt = unique(options.pt);
   end
end
if nargin < 3 || ~isfield(options,'edg') || verType == 2 % edges with tag
   edg = [];
else
   if options.edg == 0
      edg = 1:in1.meshStruct.edgTotal;
   else
      edg = unique(options.edg);
   end
   EdgesIn = in1.meshStruct.EdgesInside;
   totEdg  = size(EdgesIn,2);
end

if nargin < 3 || ~isfield(options,'q')                % quality of tr.
   C = [];
else
   C = options.q;
end
if nargin < 3 || ~isfield(options,'caxis')            % limit of color axis
   cAxis = [0 1];
else
   if length(options.caxis) == 2 && options.caxis(1) < options.caxis(2)
      cAxis = options.caxis;
   else
      fprintf(2,'Warning: plotMesh|cAxis has unsupported format! \n');
      cAxis = [0 1];
   end
end

%% ===== DATA PREPARING ===================================================
totTr = size(t,2);
totPt = size(p,2);
X   = zeros(3,totTr);
Y   = zeros(3,totTr);
Z   = zeros(3,totTr);
trC = zeros(3,totTr);
if size(p,1) == 2
   p(3,:) = 0;
end
for m = 1:totTr
   n = t(1:3,m);
   X(1:3,m) = p(1,n)';
   Y(1:3,m) = p(2,n)';
   Z(1:3,m) = p(3,n)';
   trC(:,m) = [sum(X(1:3,m))/3;sum(Y(1:3,m))/3;sum(Z(1:3,m))/3];
end
%% ===== FIGURE INITIALIZATION ============================================
thisFigure = findobj('tag',ftag);
if ~isempty(thisFigure)
   pos = get(thisFigure,'Position');
   delete(thisFigure);
else
   pos = figsize;
end
fig = figure('NumberTitle','off','Name',['plotMeshFcn ver. ' ...
   version],'DoubleBuffer','on','Units','pixels',...
   'Position',pos,'Tag','plotMesh_FIG',...
   'Color',[1 1 1],'MenuBar','figure');
ax = axes('Parent',fig,'Position',[0.07 0.07 0.86 0.86]);
hold on;

if ~isempty(figName)
   title(figName);
end
%% ===== RESULTS DEPICTION ================================================
if isempty(C)
   hndls.fill = fill3(X,Y,Z,0,'EdgeAlpha',1,'Parent',ax,...
      'FaceColor',options.FaceClr,'EdgeColor',options.EdgeClr,...
      'LineWidth',options.LineWth);
else
   hndls.fill = fill3(X,Y,Z,C,'EdgeAlpha',1,'Parent',ax,'EdgeColor',...
      options.EdgeClr,'LineWidth',options.LineWth);
   colorbar;
   caxis(cAxis);
end
for k = 1:length(tr)
   thisTr = fix(tr(k));
   if thisTr > 0 && thisTr <= totTr
      set(hndls.fill(thisTr),'EdgeColor',abs(options.EdgeClr-0.5),...
         'FaceColor',options.FaceUser);
      hndls.trTexts(k) = text(trC(1,thisTr)+0.0001,trC(2,thisTr)+0.0001,...
         trC(3,thisTr)+0.0001,num2str(thisTr),'Color','g','FontWeight',...
         'bold','HorizontalAlignment','center',...
         'VerticalAlignment','bottom','Fontsize',8);
   end
end
for k = 1:length(pt)
   thisPt = fix(pt(k));
   if thisPt > 0 && thisPt <= totPt
      hndls.ptTexts(k) = text(p(1,thisPt)+0.0001,p(2,thisPt)+0.0001,...
         p(3,thisPt)+0.0001,num2str(thisPt),'Color',[.3 .25 .9],...
         'FontWeight','bold','HorizontalAlignment','center',...
         'VerticalAlignment','bottom','Fontsize',8);
      line(p(1,thisPt),p(2,thisPt),p(3,thisPt),'LineStyle','none',...
         'Marker','o','LineWidth',2,'Color',[.3 .25 .9]);
   end
end
for n = 1:length(edg)
   thisEdg = fix(edg(:,n));
   if thisEdg > 0 && thisEdg <= totEdg
      p1 = p(:,EdgesIn(1,thisEdg));
      p2 = p(:,EdgesIn(2,thisEdg));
      xp = (p1(1)+p2(1))/2;
      yp = (p1(2)+p2(2))/2;
      zp = (p1(3)+p2(3))/2;
      hndls.edgLine(n) = line([p1(1) p2(1)],[p1(2) p2(2)],[p1(3) p2(3)],...
         'Color',options.EdgeUser,'LineWidth',2);
      hndls.edgTexts(n) = text(xp+0.0001,yp+0.0001,zp+0.0001,...
         num2str(thisEdg),'Color',options.EdgeUser,'FontWeight','bold',...
         'HorizontalAlignment','left','VerticalAlignment',...
         'bottom','Fontsize',8);
   end
end

%% ===== ADVANCED OPTIONS =================================================
[~,avgMin, minMin] = meshQual(p,t);
clear mesh;
hndls.qualText = uicontrol('Units','Pixels','Style','Text',...
   'Position',[75 50 300 20],'HorizontalAlignment','center',...
   'Parent',fig,'String',['trTotal: ' num2str(totTr) ...
   ', average qmin: ' num2str(avgMin) ', min. qmin: ' ...
   num2str(minMin)],'BackgroundColor','w');
%% ===== END OF FUNCTION ==================================================
axis('equal');
rotate3d on;
drawnow;
end

function [trTotal, qminAvq, qminMin, q] = meshQual(in1,in2)
%% meshQual: computes quality of given mesh from (p,t), Mesh.p/t
% -utility-
%
%  Syntax:
%    [trTotal qminAverage qminMinimum q] = meshQual(in1,in2);
%
% Examples:
%   meshQual(p,t);
%   meshQual(Mesh);    /Mesh.p, Mesh.t/
%   [trTotal,qminAvq,qminMin] = meshQual(p,t);
%   [trTotal,qminAvq,qminMin,q] = meshQual(p,t);
%
%    Last update: 16.5.2015
%
% Author(s): Miloslav apek, capekm6@fel.cvut.cz, 2011-2015
%
% See also plotMesh
%
% Quality calculation:
% q = 4*sqrt(3)*ar./(h1sq+h2sq+h3sq);
%   where:
%     ar = sqrt(o.*(o-a).*(o-b).*(o-c));        Triangle AREA (Heron's eq.)
%     o  = (a+b+c)/2;                           Triangle PERIMETER
%     a,b,c; a = sqrt(h1sq); b = sqrt(...); ... Triangle SIDES

if nargin == 0
   fprintf(2,'Error in meshQual: At least one input parameter in necessary!');
   trTotal = []; qminAvq = []; qminMin = []; q = [];
   return;
end
if nargin == 1 && isfield(in1,'meshStruct') && ...
      isfield(in1.meshStruct,'p') && isfield(in1.meshStruct,'t')
   p = in1.meshStruct.p;
   t = in1.meshStruct.t;
elseif nargin == 1
   p = in1.p;
   t = in1.t;
else
   p = in1;
   t = in2;
end

if size(p,1) == 2
   p(3,:) = 0;
end
trTotal = length(t); % poet trojhelnk
% Vrcholy trojhelnk
a1 = t(1,:); % BOD1
a2 = t(2,:); % BOD2
a3 = t(3,:); % BOD3
% Dlky vech stran (ale jet neodmocnno)
h3sq = (p(1,a1)-p(1,a2)).^2 + (p(2,a1)-p(2,a2)).^2 + (p(3,a1)-p(3,a2)).^2;
h1sq = (p(1,a2)-p(1,a3)).^2 + (p(2,a2)-p(2,a3)).^2 + (p(3,a2)-p(3,a3)).^2;
h2sq = (p(1,a3)-p(1,a1)).^2 + (p(2,a3)-p(2,a1)).^2 + (p(3,a3)-p(3,a1)).^2;
% Dlky stran
a = sqrt(h1sq); b = sqrt(h2sq); c = sqrt(h3sq);
% Obvod trojhelnk
o = (a+b+c)/2;
% Obsah trojhelnku (Heronv vzorec)
ar = sqrt(o.*(o-a).*(o-b).*(o-c));
% Vpoet kvality
q = 4*sqrt(3)*ar./(h1sq+h2sq+h3sq);

%                     4*sqrt(3)*a
%               q = ---------------,
%                    h1^2+h2^2+h3^2

qminAvq = sum(q)/trTotal; % prmrn Qmin
qminMin = min(q);         % prmrn kvalita meshe
end